local nData = LibStub("AceAddon-3.0"):NewAddon("nData", "AceEvent-3.0")
local Adjust = LibStub:GetLibrary("LibBasicAdjust-1.0", true)
local L = setmetatable({}, { __index = function(t,k)
	local v = tostring(k)
	rawset(t, k, v)
	return v
end })


------------------------------------------------------------------------
--	 nData Database
------------------------------------------------------------------------

local db
local defaults = {
	profile = {
		enable = true,
		
		fontNormal = 		[[Interface\Addons\nData\Media\NORMAL.ttf]],
		fontSize = 15,
			
		battleground = true,                            	-- enable 3 stats in battleground only that replace stat1,stat2,stat3.
		bag = false,										-- True = Open Backpack; False = Open All bags			
		classcolor = true,
		recountraiddps 	= false,							-- Enables tracking or Recounts Raid DPS
		
		
		-- Color Datatext	
		customcolor = { r = 1, g = 1, b = 1},				-- Color of Text for Datapanel

		-- Stat Locations
		plugins = {
			Bags			= 9,		-- show space used in bags on panel.
			System 			= 0,		-- show total memory and others systems info (FPS/MS) on panel.
			Guild 			= 4,		-- show number on guildmate connected on panel.
			Durability 		= 8,		-- show your equipment durability on panel.
			Friends 		= 6,		-- show number of friends connected.
			Spec 			= 5,		-- show your current spec on panel.
			Coordinates 	= 0,		-- show your current coords on panel.
			Professions 	= 7,		-- shows your professions and tradeskills
			Stat1 			= 1,		-- Stat Based on your Role (Avoidance-Tank, AP-Melee, SP/HP-Caster)
			Stat2 			= 3,		-- Stat Based on your Role (Armor-Tank, Crit-Melee, Crit-Caster)
			Recount 		= 2,		-- Stat Based on Recount"s DPS
			CallToArms 		= 0,		-- Show Current Call to Arms.
		}	
	}
}

------------------------------------------------------------------------
-- Constants (variables whose values are never altered):
------------------------------------------------------------------------
local GAME_LOCALE = GetLocale()
local BACKGROUND = [[Interface\DialogFrame\UI-DialogBox-Background-Dark]]
local BORDERPANEL = [[Interface\AddOns\nData\Media\UI-DialogBox-Border.blp]]


------------------------------------------------------------------------
-- Variables that point to frames or other objects:
------------------------------------------------------------------------
local nData_Datapanel, nData_StatPanelLeft, nData_StatPanelCenter, StatPanelRight, BGPanel
local classColor, currentFightDPS
local pluginOrder = {} -- add this
local function sortPlugins(a, b)
	if not a then return true end
	if not b then return false end
	local orderA, orderB = db.plugins[a] or 100, db.plugins[b] or 100
	if orderA == orderB then
		return strupper(a) < strupper(b)
	else
		return orderA < orderB
	end
end

------------------------------------------------------------------------
--	 nData Functions
------------------------------------------------------------------------

function nData:SetDataPanel()

	nData_Datapanel = CreateFrame("Frame", nil, UIParent)

	nData_Datapanel:SetPoint("BOTTOM", UIParent, 0, 0)
	nData_Datapanel:SetWidth(1200)
	nData_Datapanel:SetFrameLevel(1)
	nData_Datapanel:SetHeight(35)
	nData_Datapanel:SetFrameStrata("LOW")
	nData_Datapanel:SetBackdrop({ bgFile = BACKGROUND, edgeFile = BORDERPANEL, edgeSize = 25, insets = { left = 5, right = 5, top = 5, bottom = 5 } })
	nData_Datapanel:SetBackdropColor(0, 0, 0, 1)

	-- Hide Panels When in a Vehicle or Pet Battle
	nData_Datapanel:RegisterUnitEvent("UNIT_ENTERING_VEHICLE", "player")
	nData_Datapanel:RegisterUnitEvent("UNIT_EXITED_VEHICLE", "player")
	nData_Datapanel:RegisterUnitEvent("PET_BATTLE_OPENING_START")
	nData_Datapanel:RegisterUnitEvent("PET_BATTLE_CLOSE")
	nData_Datapanel:RegisterUnitEvent("PLAYER_ENTERING_WORLD")

	if Adjust then	
		nData_Datapanel:SetScript("OnEvent", function(self, event, ...)
			if event == "UNIT_ENTERING_VEHICLE" or event == "PET_BATTLE_OPENING_START" then
				Adjust:Unregister(nData_Datapanel)
				self:Hide()
			elseif event == "UNIT_EXITED_VEHICLE" or event == "PET_BATTLE_CLOSE" then
				Adjust:RegisterBottom(nData_Datapanel)
				self:Show()
			else
				Adjust:RegisterBottom(nData_Datapanel)
				self:SetShown(not C_PetBattles.IsInBattle() and not UnitHasVehicleUI("player"))
			end
		end)	
	end

end

function nData:SetStatPanelLeft()
	nData_StatPanelLeft = CreateFrame("Frame", nil, nData_Datapanel)
	nData_StatPanelLeft:SetPoint("LEFT", nData_Datapanel, 5, 0)
	nData_StatPanelLeft:SetHeight(35)
	nData_StatPanelLeft:SetWidth(1200 / 3)
	nData_StatPanelLeft:SetFrameStrata("MEDIUM")
	nData_StatPanelLeft:SetFrameLevel(1)
	
	nData_StatPanelLeft:RegisterUnitEvent("PLAYER_ENTERING_WORLD")
	nData_StatPanelLeft:SetScript("OnEvent", function(self, event, ...)
		if event == 'PLAYER_ENTERING_WORLD' then
			local inInstance, instanceType = IsInInstance()
			if inInstance and (instanceType == 'pvp') then			
				self:Hide()
			else
				self:Show()
			end
		end
	end)
end

function nData:SetStatPanelCenter()	
	nData_StatPanelCenter = CreateFrame("Frame", nil, nData_Datapanel)
	nData_StatPanelCenter:SetPoint("CENTER", nData_Datapanel, 0, 0)
	nData_StatPanelCenter:SetHeight(35)
	nData_StatPanelCenter:SetWidth(1200 / 3)
	nData_StatPanelCenter:SetFrameStrata("MEDIUM")
	nData_StatPanelCenter:SetFrameLevel(1)
end

function nData:SetStatPanelRight()
	nData_StatPanelRight = CreateFrame("Frame", nil, nData_Datapanel)
	nData_StatPanelRight:SetPoint("RIGHT", nData_Datapanel, -5, 0)
	nData_StatPanelRight:SetHeight(35)
	nData_StatPanelRight:SetWidth(1200 / 3)
	nData_StatPanelRight:SetFrameStrata("MEDIUM")
	nData_StatPanelRight:SetFrameLevel(1)
end

function nData:SetBattlegroundPanel()

	-- Create BGPanel
	local BGPanel = CreateFrame("Frame", nil, nData_Datapanel)
	BGPanel:SetAllPoints(nData_StatPanelLeft)
	BGPanel:SetFrameStrata("LOW")
	BGPanel:SetFrameLevel(1)
	
	--Map IDs
	local WSG = 443
	local TP = 626
	local AV = 401
	local SOTA = 512
	local IOC = 540
	local EOTS = 482
	local TBFG = 736
	local AB = 461

	local bgframe = BGPanel
	bgframe:SetScript('OnEnter', function(self)
		local numScores = GetNumBattlefieldScores()
		for i=1, numScores do
			local name, killingBlows, honorableKills, deaths, honorGained, faction, race, class, classToken, damageDone, healingDone, bgRating, ratingChange = GetBattlefieldScore(i)
			if ( name ) then
				if ( name == UnitName('player') ) then
					GameTooltip:SetOwner(self, 'ANCHOR_TOPLEFT', 0, 4)
					GameTooltip:ClearLines()
					GameTooltip:SetPoint('BOTTOM', self, 'TOP', 0, 1)
					GameTooltip:ClearLines()
					GameTooltip:AddLine("Stats for : "..hexa..name..hexb)
					GameTooltip:AddLine' '
					GameTooltip:AddDoubleLine("Killing Blows:", killingBlows,1,1,1)
					GameTooltip:AddDoubleLine("Honorable Kills:", honorableKills,1,1,1)
					GameTooltip:AddDoubleLine("Deaths:", deaths,1,1,1)
					GameTooltip:AddDoubleLine("Honor Gained:", format('%d', honorGained),1,1,1)
					GameTooltip:AddDoubleLine("Damage Done:", damageDone,1,1,1)
					GameTooltip:AddDoubleLine("Healing Done:", healingDone,1,1,1)
					--Add extra statistics to watch based on what BG you are in.
					if curmapid == WSG or curmapid == TP then 
						GameTooltip:AddDoubleLine("Flags Captured:",GetBattlefieldStatData(i, 1),1,1,1)
						GameTooltip:AddDoubleLine("Flags Returned:",GetBattlefieldStatData(i, 2),1,1,1)
					elseif curmapid == EOTS then
						GameTooltip:AddDoubleLine("Flags Captured:",GetBattlefieldStatData(i, 1),1,1,1)
					elseif curmapid == AV then
						GameTooltip:AddDoubleLine("Graveyards Assaulted:",GetBattlefieldStatData(i, 1),1,1,1)
						GameTooltip:AddDoubleLine("Graveyards Defended:",GetBattlefieldStatData(i, 2),1,1,1)
						GameTooltip:AddDoubleLine("Towers Assaulted:",GetBattlefieldStatData(i, 3),1,1,1)
						GameTooltip:AddDoubleLine("Towers Defended:",GetBattlefieldStatData(i, 4),1,1,1)
					elseif curmapid == SOTA then
						GameTooltip:AddDoubleLine("Demolishers Destroyed:",GetBattlefieldStatData(i, 1),1,1,1)
						GameTooltip:AddDoubleLine("Gates Destroyed:",GetBattlefieldStatData(i, 2),1,1,1)
					elseif curmapid == IOC or curmapid == TBFG or curmapid == AB then
						GameTooltip:AddDoubleLine("Bases Assaulted:",GetBattlefieldStatData(i, 1),1,1,1)
						GameTooltip:AddDoubleLine("Bases Defended:",GetBattlefieldStatData(i, 2),1,1,1)
					end					
					GameTooltip:Show()
				end
			end
		end
	end) 
	bgframe:SetScript('OnLeave', function(self) GameTooltip:Hide() end)

	local plugin = CreateFrame('Frame', nil)
	plugin:EnableMouse(true)

	local Text1  = BGPanel:CreateFontString(nil, 'OVERLAY')
	Text1:SetFont(db.fontNormal, db.fontSize,'THINOUTLINE')
	Text1:SetPoint('LEFT', BGPanel, 30, 0)
	Text1:SetHeight(nData_Datapanel:GetHeight())

	local Text2  = BGPanel:CreateFontString(nil, 'OVERLAY')
	Text2:SetFont(db.fontNormal, db.fontSize,'THINOUTLINE')
	Text2:SetPoint('CENTER', BGPanel, 0, 0)
	Text2:SetHeight(nData_Datapanel:GetHeight())

	local Text3  = BGPanel:CreateFontString(nil, 'OVERLAY')
	Text3:SetFont(db.fontNormal, db.fontSize,'THINOUTLINE')
	Text3:SetPoint('RIGHT', BGPanel, -30, 0)
	Text3:SetHeight(nData_Datapanel:GetHeight())

	local int = 2
	local function Update(self, t)
		int = int - t
		if int < 0 then
			local dmgtxt
			RequestBattlefieldScoreData()
			local numScores = GetNumBattlefieldScores()
			for i=1, numScores do
				local name, killingBlows, honorableKills, deaths, honorGained, faction, race, class, classToken, damageDone, healingDone, bgRating, ratingChange = GetBattlefieldScore(i)
				if healingDone > damageDone then
					dmgtxt = ("Healing : "..hexa..healingDone..hexb)
				else
					dmgtxt = ("Damage : "..hexa..damageDone..hexb)
				end
				if ( name ) then
					if ( name == PLAYER_NAME ) then
						Text2:SetText("Honor : "..hexa..format('%d', honorGained)..hexb)
						Text1:SetText(dmgtxt)
						Text3:SetText("Killing Blows : "..hexa..killingBlows..hexb)
					end   
				end
			end 
			int  = 0
		end
	end

	--hide text when not in an bg
	local function OnEvent(self, event)
		if event == 'PLAYER_ENTERING_WORLD' then
			local inInstance, instanceType = IsInInstance()
			if inInstance and (instanceType == 'pvp') then			
				bgframe:Show()
			else
				Text1:SetText('')
				Text2:SetText('')
				Text3:SetText('')
				bgframe:Hide()
			end
		end
	end

	plugin:RegisterEvent('PLAYER_ENTERING_WORLD')
	plugin:SetScript('OnEvent', OnEvent)
	plugin:SetScript('OnUpdate', Update)
	Update(plugin, 10)
end

function nData:GetTooltipAnchor()
	local _, y = GetCursorPosition()
	if (y * 2) > UIParent:GetHeight() then
		return "ANCHOR_TOP", 1, -3
	end
end
--[[function nData:DataTextTooltipAnchor(self)
	local panel = self:GetParent()
	local anchor = 'GameTooltip'
	local xoff = 1
	local yoff = 3
	
	
	for _, panel in pairs ({
		nData_StatPanelLeft,
		nData_StatPanelCenter,
		nData_StatPanelRight,
	})	do
		anchor = 'ANCHOR_TOP'
	end	
	return anchor, panel, xoff, yoff
end]]

function nData:PlacePlugin(plugin, i)
	local area = math.ceil(i / 3)
	
	local parent
	if area == 1 then
		parent = nData_StatPanelLeft
	elseif area == 2 then
		parent = nData_StatPanelCenter
	elseif area == 3 then
		parent = nData_StatPanelRight
	end
	if not parent then
		return -- somehow you passed a value > 9
	end

	plugin:SetParent(parent)
	plugin:SetPoint("TOP")
	plugin:SetPoint("BOTTOM")
	if i == (area * 3 - 2) then
		plugin:SetPoint("LEFT", 30, 0)
	else
		plugin:SetPoint("LEFT", parent[i-1], "RIGHT", 10, 0) -- adjust for padding between plugins
	end
	parent[i] = plugin
end	

function nData:OnInitialize()
	-- Assuming the .toc says ## SavedVariables: MyAddonDB
	self.db = LibStub("AceDB-3.0"):New("nDataDB", defaults, true)
	db = self.db.profile
	
	self:SetUpOptions();

	if not db.enable then return end
	
	self:CreatePanel() -- factor this giant blob out into its own function to keep things clean
	
end

function nData:OnEnable()
	
	-- Rather than hardcode all the possible plugins here just use a nice table that you can add/remove stuff to much more easily. Scroll to the bottom to see it.
	self.plugins = {}
	for name, state in pairs(db.plugins) do
		if state > 0 then
			local func = self.pluginConstructors[name]
			self.plugins[name] = func and func() or nil
			tinsert(pluginOrder, name)
		end
	end

	sort(pluginOrder)

	for i = 1, #pluginOrder do
		local name = pluginOrder[i]
		db.plugins[name] = i -- fix any weirdness
		self:PlacePlugin(self.plugins[name], i)		
	end	
	
		-- Move the tooltip above the Actionbar
	hooksecurefunc('GameTooltip_SetDefaultAnchor', function(self)
		self:SetPoint('BOTTOMRIGHT', UIParent, -95, 135)
	end)
	
	-- no need to make a separate frame to handle events, your module object already does this!
	self:UpdatePlayerRole()
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED", "UpdatePlayerRole")	
	self:Refresh()
end

function nData:Refresh()

	db = self.db.profile

	if InCombatLockdown() then
		return self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnEnable")
	end
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	
end

function nData:SetFontString(parent, file, size, flags)
	local fs = parent:CreateFontString(nil, "OVERLAY")
	fs:SetFont(file, size, flags)
	fs:SetJustifyH("LEFT")
	fs:SetShadowColor(0, 0, 0)
	fs:SetShadowOffset(1.25, -1.25)
	return fs
end

function nData:CreatePanel()

	if nData_Datapanel then return end -- already done
	
	-- Setup the Panels
	self:SetDataPanel()
	self:SetStatPanelLeft()
	self:SetStatPanelCenter()
	self:SetStatPanelRight()
	self:SetBattlegroundPanel()

end


function nData:UpdatePlayerRole()
	local spec = GetSpecialization()
	local specRole = GetSpecializationRole(spec) -- no need for a giant table that must be maintained by hand
	if specRole == "TANK" then
		playerRole = "Tank"
	elseif specRole == "HEALER" then
		playerRole = "Caster"
	elseif specRole == "DAMAGER" then
		if UnitPowerType("player") == SPELL_POWER_MANA then
			playerRole = "Caster"
		else
			playerRole = "Melee"
		end
	else
		playerRole = nil -- no spec
	end
end

function nData:SetUpOptions()
	local options = self:GetOptions()

	-- @PHANX: add this to your options table instead of registering it separately:
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("nData", options)

	local Dialog = LibStub("AceConfigDialog-3.0")

	-- Use the "welcome" panel as the main one:
	self.optionsFrame = Dialog:AddToBlizOptions("nData", "nData", nil, "datatext")

	-- Add the profile panel last:
	Dialog:AddToBlizOptions("nData", options.args.profile.name, "nData", "profile")

end

------------------------------------------------------------------------
-- Stat Creation
------------------------------------------------------------------------
-- This can be simplified hugely by using a table, instead of a bunch of if/end blocks.

nData.pluginConstructors = {}
		
------------------------------------------------------------------------
--	 nData Options
------------------------------------------------------------------------

local options
function nData:GetOptions()
	if options then
		return options
	end

	local statposition = {
		["0"] = L["Not Shown"],
		["1"] = L["Position #1"],
		["2"] = L["Position #2"],
		["3"] = L["Position #3"],
		["4"] = L["Position #4"],
		["5"] = L["Position #5"],
		["6"] = L["Position #6"],
		["7"] = L["Position #7"],
		["8"] = L["Position #8"],
		["9"] = L["Position #9"],
	}
	
	options = {
		type = "group",
		name = L["nData"],
		childGroups = "tree",
		get = function(info) return db[ info[#info] ] end,
		set = function(info, value) db[ info[#info] ] = value;  self:Refresh() end,
		args = {
			datatext = { 
				type = "group",
				order = 1,
				name = "nData",
				args = {			
					reloadUI = {
						type = "execute",
						name = "Reload UI",
						desc = " ",
						order = 0,
						func = 	function()
							ReloadUI()
						end,
					},
					Text = {
						type = "description",
						order = 0,
						name = "When changes are made a reload of the UI is needed.",
						width = "full",
					},
					Text1 = {
						type = "description",
						order = 1,
						name = " ",
						width = "full",
					},
					time24 = {
						type = "toggle",
						order = 2,
						name = L["24-Hour Time"],
						desc = L["Display time datapanel on a 24 hour time scale"],
						disabled = function() return  not db.enable end,
					},
					bag = {
						type = "toggle",
						order = 2,
						name = L["Bag Open"],
						desc = L["Checked opens Backpack only, Unchecked opens all bags."],
						disabled = function() return  not db.enable end,
					},
					battleground = {
						type = "toggle",
						order = 2,
						name = L["Battleground Text"],
						desc = L["Display special datapanels when inside a battleground"],
						disabled = function() return  not db.enable end,
					},
					recountraiddps = {
						type = "toggle",
						order = 2,
						name = L["Recount Raid DPS"],
						desc = L["Display Recount's Raid DPS (RECOUNT MUST BE INSTALLED)"],
						disabled = function() return  not db.enable end,
					},
					fontSize = {
						type = "range",
						order = 5,						
						name = L["Game Font Size"],
						desc = L["Controls the Size of the Game Font"],
						disabled = function() return not db.enable end,
						min = 0, max = 30, step = 1,
					},
					classcolor = {
						type = "toggle",					
						order = 2,
						name = L["Class Color"],
						desc = L["Use your classcolor for border and some text color."],
						disabled = function() return not db.enable end,
					},
					DataGroup = {
						type = "group",
						order = 6,
						name = L["Text Positions"],
						guiInline  = true,
						get = function(info)
							return tostring(db.plugins[ info[#info] ])
						end,
						set = function(info, value)
							value = tonumber(value)
							local name = info[#info]
							-- Remove this plugin from the order:
							for i = 1, #pluginOrder do
								if pluginOrder[i] == name then
									tremove(pluginOrder, i)
								end
							end
							-- Re-sort the remaining plugins:
							sort(pluginOrder, sortPlugins)

							if value > 0 then
								-- Put this plugin back in the desired place,
								-- but only if it's enabled:
								tinsert(pluginOrder, name, value)
							else
								-- Mark it disabled in the db:
								db.plugins[name] = 0
							end

							-- Now re-place all the plugins, and update
							-- the db values according to the new order:
							for i = 1, #pluginOrder do
								local name = pluginOrder[i]
								db.plugins[name] = i
								self:PlacePlugin(self.plugins[name], i)
							end
						end,
						args = {},
					},					
				},
			},
		},
	}
	local t = options.args.datatext.args.DataGroup.args
	for name in pairs(self.pluginConstructors) do
		t[name] = {
			type = "select",
			name = name,
			values = statposition,
		}
	end

	return options
end